;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
breed [self-links self-link] ;;for linking with oneself
extensions [matrix] 

;; each person has wealth and gives/receives to/from the same or dif. group            
turtles-own [seNo rankList pre-Token token start-token to-in-group from-in-group to-out-group from-out-group group-number fair-value ]  

;;Lots of global variable due to the nature of the output - enable output of the values from any module
globals[GameName start-token-list rankMatrix mainFrom mainTo listFrom listTo seqNoList Empathy-matrix self-interest-matrix group-matrix fairness-matrix
  vicinity-matrix reciprocity-matrix random-matrix change-vector number-of-persons Gcounter ] 
;///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


;;display tokens beside each person.
;///////////////////////////////////////////////////
to display-labels
  ask turtles [ set label "" ]
  if show-token? [
    ask turtles [ set label token ]
  ]
end
;///////////////////////////////////////////////////

;;chooses who to give token based on the empathy matrix. this is based on the highest value in the column of the empathy matrix
;////////////////////////////////////////////////////
to choose-a-person
  let count1 0
  repeat number-of-persons[
    let count2 0
    let likely-who matrix:get Empathy-matrix count2 count1
    let likely-who-index count2
    repeat number-of-persons[
      ifelse (matrix:get Empathy-matrix count2 count1 > likely-who)[
        set likely-who matrix:get Empathy-matrix count2 count1 
        set likely-who-index count2
        set count2 (count2 + 1)
      ]
      [
        ifelse(matrix:get Empathy-matrix count2 count1 = likely-who)[
          ifelse([token] of turtle count2 < [token] of turtle likely-who-index)[
            set likely-who matrix:get Empathy-matrix count2 count1 
            set likely-who-index count2
            set count2 (count2 + 1)
          ]
          [
            ifelse([token] of turtle count2 = [token] of turtle likely-who-index)[
              ifelse(random 2 = 0)[
                set likely-who matrix:get Empathy-matrix count2 count1 
                set likely-who-index count2
                set count2 (count2 + 1)
              ]
              [
                set count2 (count2 + 1)
              ]
            ]
            [
              set count2 (count2 + 1)
            ]
          ]
        ]
        [
          set count2 (count2 + 1)
        ]
        
      ]
    ]
    give-out-token count1 turtle likely-who-index
    set count1 (count1 + 1)
  ] 
end
;//////////////////////////////////////////////////


;;give token to a specific person
to give-out-token [current-who a-person ]
  set listFrom lput current-who listFrom
  set listTo lput ([who] of a-person) listTo
  ifelse ( turtle current-who = a-person)[
    
    ask turtle current-who [
      if(token > 0)[
        
        ;; use breed of turtles to create self link since Netlogo does not allow self-link
        hatch-self-links 1[
          set token " "
          set color red
          set size 0.7
          ask self-link who [
            lt 90
            pen-down
            repeat 25[
              arc-forward-by-dist 12
            ]
          ]
        ]
        set from-in-group (from-in-group + 1)
        set to-in-group (to-in-group + 1)
        ask self-links [die]
        set change-vector replace-item ([who] of a-person) change-vector (1 + item ([who] of a-person) change-vector)
      ]
    ]
  ]
  [ 
    ask turtle current-who [
      if(token > 0)[
        ifelse([breed] of turtle current-who = [breed] of a-person)[
          set to-in-group (to-in-group + 1)
          ask a-person[
            set from-in-group (from-in-group + 1)
          ]
        ]
        [
          set to-out-group (to-out-group + 1)
          ask a-person[
            set from-out-group (from-out-group + 1)
          ]
        ]
        create-link-to a-person [set color red]
        set change-vector replace-item ([who] of a-person) change-vector (1 + item ([who] of a-person) change-vector)
      ]
    ]
  ]
end
;///////////////////////////////////////////////////

;;set the model in motion
;///////////////////////////////////////////////////
to go
  set listFrom[]
  set listTo[]
  set rankMatrix[]
  initialize-change-vector
  make-empathy-matrix
  compute-rank empathy-matrix
  clear-links
  clear-drawing
  initialize-exchanges
  choose-a-person
  update-token
  display-labels
end
;//////////////////////////////////////////////////////


;; create self link
;////////////////////////////////////////////////////// 
to arc-forward-by-dist [dist]          ;; calculate how much of an angle we'll be turning through
  let theta dist * 180 / (pi * 2)      ;; (essentially converting radians to degrees)
  rt theta / 2                         ;; turn to face the next point we're going to
  fd dist                              ;; go there
  rt theta / 2                         ;; turn to face tangent to the circle
end
;///////////////////////////////////////////////////


;;compute the self-interest matrix. Simply an identity matrix
;////////////////////////////////////////////////// 
to make-self-interest-matrix
  set self-interest-matrix matrix:make-identity number-of-persons
end
;/////////////////////////////////////////////////


;;compute the group matrix. set a position to 1 if both participant i and j are in the same group
;/////////////////////////////////////////////////
to make-group-matrix
  set group-matrix matrix:make-constant number-of-persons number-of-persons 0
  let count1 0
  let count2 0
  repeat number-of-persons [
    set count2 0
    repeat number-of-persons[
      let gn2 0
      let gn1 0
      ask turtle count1 [set gn1 group-number]
      ask turtle count2 [set gn2 group-number]
      if ( gn1 = gn2)[
        set group-matrix matrix:set-and-report group-matrix count1 count2 1
      ]
      set count2 (count2 + 1)
    ]
    set count1 (count1 + 1 )
  ]
  ;  print group-matrix
end
;////////////////////////////////////////////////


;;compute the matrix for adding random numbers
;/////////////////////////////////////////////////////
to make-random-matrix
  set random-matrix matrix:make-constant number-of-persons number-of-persons 0
  let counter 0
  repeat number-of-persons [
    let gen-number random number-of-persons
    if not Random-self-given?[
      while[gen-number = counter][
        set gen-number random number-of-persons
      ]
    ]
    matrix:set random-matrix gen-number counter random-value
    set counter (counter + 1)
  ]
 ;  show word "random matrix" random-matrix
end
;/////////////////////////////////////////////////////



;;compute the fairness matrix. Perticipant with list token gets the highest value
;//////////////////////////////
to make-fairness-matrix
  set fairness-matrix matrix:make-constant number-of-persons number-of-persons 0
  let count1 number-of-persons
  let count2 count1
  let counter 0
  let fair-list1 []
  let fair-value-list []
  let fair-list sort-on [token] turtles
  foreach fair-list [ ask ? [set fair-list1 lput token fair-list1] ]
  set fair-value-list lput count1 fair-value-list
  repeat number-of-persons - 1 [ 
    ifelse(item counter fair-list1 = item (counter + 1) fair-list1) [
      set fair-value-list lput count1 fair-value-list
      set count2 (count2 - 1)
    ][
    set count2 (count2 - 1)
    set count1 count2
    set fair-value-list lput count1 fair-value-list
    ]
    set counter (counter + 1)
  ]
  (foreach fair-list fair-value-list [ ask ?1 [ set fair-value ?2]])
  
  set count1 0
  let fairnesList[]
  repeat number-of-persons [
    set count2 0
    repeat number-of-persons[
      let value [fair-value] of turtle count1
      set fairness-matrix matrix:set-and-report fairness-matrix count1 count2 value
      set fairnesList lput value fairnesList
      set count2 (count2 + 1)
    ]
    set count1 (count1 + 1 )
  ]
  let maxFairness max fairnesList
  set fairness-matrix matrix:times-scalar fairness-matrix (1 / maxFairness)
end
;/////////////////////////////////////////////

;;compute the reciprocity matrix. set the matrix position ij to 1 if i gave to j in the previous round
;/////////////////////////////////////////////
to make-reciprocity-matrix
  set reciprocity-matrix matrix:make-constant number-of-persons number-of-persons 0
  let reciprocate-to []
  let reciprocate-from []
  let self-reciprocate []
  ask links [ 
    set reciprocate-to lput ([who] of end1) reciprocate-to
    set reciprocate-from lput ([who] of end2) reciprocate-from
  ]
  let counter 0
  repeat number-of-persons [ 
    if not (member? counter reciprocate-to) [                              
      set self-reciprocate lput counter self-reciprocate
    ]
    set counter (counter + 1)
  ] 
  (foreach  reciprocate-to reciprocate-from [
    set reciprocity-matrix matrix:set-and-report reciprocity-matrix ?1 ?2 1
  ])
  foreach self-reciprocate [
    if ([token] of turtle ? > 0)[
      set reciprocity-matrix matrix:set-and-report reciprocity-matrix ? ? 0
    ]
  ]
end 
to make-vicinity-matrix2
  set vicinity-matrix matrix:make-constant (length seqNoList) (length seqNoList) 0
  let distanceList []
  let counter1 0
  repeat length seqNoList[
    let counter2 0
    repeat length seqNoList[
      let aDistance compute-distance1 counter1 counter2 turtle counter1 turtle counter2
      set distanceList lput aDistance distanceList
      set counter2 (counter2 + 1)
    ]
    set counter1 counter1 + 1
  ]
  let maxDistance max distanceList
  
  set counter1 0
  repeat length seqNoList[
    let counter2 0
    repeat length seqNoList[
      let disValue matrix:get vicinity-matrix counter1 counter2
      if(disValue != 0)[
        let computedValue (1 - (disValue / maxDistance))
        set vicinity-matrix matrix:set-and-report vicinity-matrix counter1 counter2 computedValue
      ]
      set counter2 (counter2 + 1)
    ]
    set counter1 counter1 + 1
  ]
  ;; print vicinity-matrix
  
end
;////////////////////////////////////////////

;compute the distance between participant t1 and t2 and set this appropriately in the vecinity matrix
;///////////////////////////////////////////
to-report compute-distance1 [c1 c2 t1 t2]
  let aDistance 0
  ask t1[
    set aDistance round distance t2
    set vicinity-matrix matrix:set-and-report vicinity-matrix c1 c2 aDistance
  ]
  report aDistance  
end
;//////////////////////////////////////////


;compute the empathy matrix
;////////////////////////////////////////////
to make-empathy-matrix
 
    make-random-matrix
    make-group-matrix
    make-self-interest-matrix
    make-fairness-matrix
    make-reciprocity-matrix
    make-vicinity-matrix2
    set reciprocity-matrix matrix:times-scalar reciprocity-matrix reciprocity-weight
    set fairness-matrix matrix:times-scalar fairness-matrix fairness-weight
    set group-matrix matrix:times-scalar group-matrix group-weight
    set self-interest-matrix matrix:times-scalar self-interest-matrix self-interest-weight
    set vicinity-matrix matrix:times-scalar vicinity-matrix vicinity-weight
    let M1 (matrix:plus vicinity-matrix (matrix:plus fairness-matrix reciprocity-matrix))
    let M2 matrix:plus group-matrix self-interest-matrix
    set empathy-matrix matrix:copy (matrix:plus random-matrix (matrix:plus M1 M2))
end
;///////////////////////////////////////////////////

;;update individual tokens (done after every round)
;//////////////////////////////////////////////////
to update-token
  foreach sort-on [who] turtles[
    ask ?[
      set token (token + item who change-vector)
      if(token < 0)[
        set token 0
      ]
    ]
  ]
  
end
;///////////////////////////////////////////////

;for updating based on the file of the real live performance 
;////////////////////////////////////////////////
to re-update-token
  foreach sort-on [who] turtles[
    ask ?[
      set token pre-Token 
    ]
  ]
end
;////////////////////////////////////////////

;used to keep track of received tokens
;///////////////////////////////////  
to initialize-change-vector
 set change-vector n-values number-of-persons [-1]
  foreach sort-on [who] turtles[
    ask ?[
      set rankList n-values (length seqNoList) [length seqNoList]
      if(token = 0)[
        set change-vector replace-item who change-vector 0
      ]
    ]
  ]
end
;///////////////////////////////////////////

to-report use-range
  let RangeList[]
  let counter 0
  ;get the list of fairness weight
  let fairnessRangesList[]
  ifelse set-range? [
    set counter S-fairness
    while[counter <= E-fairness][
      set fairnessRangesList lput counter fairnessRangesList
      set counter counter + I-fairness
    ]
  ]
  [
    set fairnessRangesList lput fairness-weight fairnessRangesList
  ]
  set RangeList lput fairnessRangesList RangeList
  
  ;;get the list of reciprocity weights
  let reciprocityRangesList[]
  ifelse set-range?[
    set counter S-reciprocity
    while[counter <= E-reciprocity][
      set reciprocityRangesList lput counter reciprocityRangesList
      set counter counter + I-reciprocity
    ]
  ]
  [
    set reciprocityRangesList lput reciprocity-weight reciprocityRangesList
  ]
  set RangeList lput reciprocityRangesList RangeList
  
  ;get the list of group weigths
  let groupRangesList[]
  ifelse set-range?[
    set counter S-group
    while[counter <= E-group][
      set groupRangesList lput counter groupRangesList
      set counter counter + I-group
    ]
  ]
  [
    set groupRangesList lput group-weight groupRangesList
  ]
  set RangeList lput groupRangesList RangeList
  
  ;get the list of self-interest weights
  let self-interestRangesList[]
  ifelse set-range? [
    set counter S-self-interest
    while[counter <= E-self-interest][
      set self-interestRangesList lput counter self-interestRangesList
      set counter counter + I-self-interest
    ]
  ]
  [
    set self-interestRangesList lput self-interest-weight self-interestRangesList
  ]
  set RangeList lput self-interestRangesList RangeList
  
  ;get the list of vicinity weights
  let vicinityRangesList[]
  ifelse set-range?[
    set counter S-vicinity
    while[counter <= E-vicinity][
      set vicinityRangesList lput counter vicinityRangesList
      set counter counter + I-vicinity
    ]
  ]
  [
    set vicinityRangesList lput vicinity-weight vicinityRangesList
  ]
  set RangeList lput vicinityRangesList RangeList
  
  ;get the list of random weights
  let randomRangesList[]
  ifelse set-range? [
    set counter S-random
    while[counter <= E-random][
      set randomRangesList lput counter randomRangesList
      set counter counter + I-random
    ]
  ]
  [
    set randomRangesList lput random-value randomRangesList
  ]
  set RangeList lput randomRangesList RangeList
  report RangeList
end 


;////////////////////////////////////////
;keep memory of the token before starting each round 
to initialize-exchanges
  foreach sort-on [who] turtles[
    ask ?[
      set start-token token
      set pre-token token
    ]
  ]
end
;//////////////////////////////////////// 

;Read the file for the real live activities. store the neccessary data in different lists for further use (a little bit tricky to follow)
;////////////////////////////////////////////    
to file-read-for-conversion [file]
  if is-string? file[
    file-open file
    set GameName []
    let myList[]
    let myHeading[]
    let counter 0
    let FPN[] 
    let TPN[]
    let FT[]
    let WAV[]
    let FG[]
    let TN[]
    let GN[]
    let FX[]
    let FY[]
    let groupToken[]
    let nFPN 0
    let nTPN 0
    let nFT 0
    let nWAV 0
    let nFG 0
    let nTN 0
    let nGN 0
    let nFX 0
    let nFY 0
    
    
    while[not file-at-end?][
      let myLine file-read-line
      ifelse(counter = 0)[
        set myHeading string:split ";," myline 
        set counter counter + 1
        set nFPN position "fromSeqNo" myHeading
        set nTPN position "toSeqNo" myHeading
        set nFT position "tokens" myHeading
        set nWAV position "roundNo" myHeading 
        set nFG position "fromGroupNo" myHeading 
        set nTN position "trialNo" myHeading  
        set nGN position "gameName" myHeading
        set nFX position "fromX" myHeading
        set nFY position "fromY" myHeading 
      ]
      [
        set myList string:split ";," myline 
        set FPN lput (item nFPN myList) FPN
        set TPN lput (item nTPN myList) TPN
        set FT lput (item nFT myList) FT
        set WAV lput (item nWAV myList) WAV
        set FG lput (item nFG myList) FG
        set TN lput (item nTN myList) TN
        set GN lput (item nGN myList) GN
        set FX lput (item nFX myList) FX
        set FY lput (item nFY myList) FY
        set counter counter + 1 
      ]
      
    ]
    set GameName GN
    let gList []
    set seqNoList remove-duplicates FPN
    let groupList remove-duplicates sort-by < FG;read-from-string user-one-of "How many groups ?" ["1" "2" "3" "4"]
                                                ; let groupColor [red blue white yellow pink]
    foreach groupList[ 
      set gList lput read-from-string ? gList
      ;  set groupColor lput user-one-of (word "Select colour for group  " ?  ["red" "white" "blue" "yellow"] )
    ]
    ask patches [ set pcolor gray ] 
    create-turtles length seqNoList
    
    
    foreach seqNoList[
      let num read-from-string ?
      let gNo read-from-string item (num - 1)  FG
      let xPos read-from-string item (num - 1)  FX
      let yPos 0 - read-from-string item (num - 1)  FY
      ; let sqN read-from-string ?
      ask turtle (num - 1)[
        setxy xPos yPos
        set shape "person"                           ;; like human being
        set color  white
        set size 24.0                                 
        set label-color black
        set group-number gNo                  
      ]   
    ]
    set start-token-list[]
    ifelse "per Individul" = user-one-of "Set start tokens.." ["per Individul" "per group"][
      foreach seqNoList[
        ask turtle ((read-from-string ?) - 1) [ 
          set token read-from-string user-input  (word "Enter the start token for participants with seqNo " ?)
          set start-token-list lput token start-token-list
        ]
      ]
      
    ]
    [
      foreach sort-by < gList[
        set groupToken lput user-input (word "Enter the start token for group " ? " participants") groupToken
      ]
      foreach sort-by < seqNoList[ ask turtle ((read-from-string ?) - 1) [
        if (group-number = 1)[
          set token read-from-string item 0 groupToken
          set start-token-list lput token start-token-list
        ]
        if (group-number = 2) [
          set token read-from-string item 1 groupToken
          set start-token-list lput token start-token-list 
        ]
        if (group-number = 3) [ 
          set token read-from-string item 2 groupToken
          set start-token-list lput token start-token-list
        ]
        if (group-number = 4) [
          set token read-from-string item 3 groupToken
          set start-token-list lput token start-token-list
        ]
      ]
      ] 
    ]
    set number-of-persons length seqNoList
    reset-ticks
    display-labels
    file-close
    
    ;/////////////////////////////////////////////////////////////
    let periodCount[]
    let positionCount[]
    let checker 1
    if( length TN > 1)[
      repeat length TN - 1[
        if(item (checker - 1) TN != item checker TN)[
          set periodCount lput (item (checker - 1) WAV ) periodCount    ; tracks the number of rounds
          set positionCount lput  checker positionCount                 ; tracks the position counting from 1
        ]
        set checker checker + 1
      ]
      set periodCount lput (item (checker - 1) WAV ) periodCount
      set positionCount lput  checker positionCount
    ]
    ;/////////////////////////////////////////////////////////////////
    
    let distinctTN remove-duplicates TN
    let fileNames[]
    set mainFrom[]
    set MainTo[]
    let outTrack 0 
    let repeatCount 0
    let fileTracker 0
    repeat length distinctTN[
      ifelse (fileTracker < 1)[
        set repeatCount (item 0 positionCount)
      ]
      [
        set repeatCount ((item fileTracker positionCount) - (item (fileTracker - 1) positionCount))
      ] 
      let giveMain[]
      let receiveMain[]
      let giveSub[]
      let receiveSub[]
      let WAVc -1
      repeat repeatCount[
        if(WAVc = -1)[set WAVc read-from-string (item outTrack WAV)]
        if(WAVc != read-from-string (item outTrack WAV))[
          set giveMain lput giveSub giveMain
          set receiveMain lput receiveSub receiveMain
          set receiveSub[]
          set giveSub[]
          set WAVc read-from-string (item outTrack WAV)
        ]
        set giveSub lput (item outTrack FPN) giveSub
        set receiveSub lput (item outTrack TPN) receiveSub
        set outTrack outTrack + 1
      ]
      set giveMain lput giveSub giveMain
      set receiveMain lput receiveSub receiveMain
      set mainFrom lput giveMain mainFrom
      set mainTo lput receiveMain mainTo
      ;   use-window  giveMain receiveMain
      file-close
      set fileTracker fileTracker + 1
    ]
  ]   
end

to reset-to-start-token
  foreach sort-on [who] turtles[
    ask ? [ set token item ([who] of ?) start-token-list]
  ]
end

to go1
 ; setupFiles
  let file1 (word "EmpMatrix_Comparison_"item 0 GameName"_RoundDetails.csv")
  if file-exists? file1[
    file-close 
    file-delete file1
  ]
  let file2 (word "EmpMatrix_Comparison_"item 0 GameName"_RoundSummary.csv")
  if file-exists? file2[
    file-close 
    file-delete file2
  ]
  file-open file1
  file-print ( "Game Name , Trial No.  ,  Round No.  , SeqNo ,  Prediction accuracy ,  Predicted Rank ,  Scaled Prediction Rank  , Fairness weight  ,  Reciprocity weight ,  Group Weight ,self Interest weight  ,  vicinity weight , Random value")
  file-close
  file-open file2
  file-print ( "Game Name , Trial No.  ,  Round No.  , Average Rank ,  Percent Correct, Fairness weight  ,  Reciprocity weight ,  Group Weight ,self Interest weight  ,  vicinity weight , Random value")
  file-close
  let RangeList use-range
  foreach item 0 RangeList[
    let fairnessWeight ?
    set fairness-Weight fairnessWeight
    foreach item 1 RangeList[
      let reciprocityWeight ?
      set reciprocity-Weight reciprocityWeight
      foreach item 2 RangeList[
        let groupWeight ?
        set group-Weight groupWeight
        foreach item 3 RangeList[
          let selfInterestWeight ?
          set self-Interest-Weight selfInterestWeight
          foreach item 4 RangeList[
            let vicinityWeight ?
            set vicinity-Weight vicinityWeight
            foreach item 5 RangeList[
              let randomWeight ?
              set random-value randomWeight
              let trialCount 1
              let totalGameCount 0
              (foreach mainFrom mainTo [
                ;///trial boudary
                reset-to-start-token
                let roundCount 1
                let roundTrue 0
                let roundFalse 0
                
                (foreach ?1 ?2 [
                   let CorrectCount 0
                   let averageRank []
                 file-open file1
                  ;///round boundary
                  go
                  ;////compares the agents simulation to human
                  ( foreach ?1 ?2 listFrom listTo [
                    ifelse (((read-from-string ?1) - 1) = ?3 and ((read-from-string ?2) - 1) = ?4)[
                       file-print (word (item totalGameCount GameName)", "trialCount", " roundCount ", "(read-from-string ?1)" , " 1 " , "[item ((read-from-string ?2) - 1) rankList] of turtle ?3","(1 - ((([item ((read-from-string ?2) - 1) rankList] of turtle ?3) - 1)/(length seqNoList - 1)))", " fairnessWeight ", "reciprocityWeight ", " groupWeight ", "selfInterestWeight", "vicinityWeight", "randomWeight)
                       set CorrectCount CorrectCount + 1
                    ]
                    [
                    file-print (word (item totalGameCount GameName)", "trialCount", " roundCount ", "(read-from-string ?1)" , " 0 " , "[item ((read-from-string ?2) - 1) rankList] of turtle ?3", "(1 - ((([item ((read-from-string ?2) - 1) rankList] of turtle ?3) - 1)/(length seqNoList - 1)))", " fairnessWeight ", "reciprocityWeight ", " groupWeight ", "selfInterestWeight", "vicinityWeight", "randomWeight)
                    ]
                        set totalGameCount totalGameCount + 1
                          
                  ])
                   foreach sort-on [who] turtles[
                     set averageRank lput ((sum [rankList] of ?)/ (length [rankList] of ?)) averageRank
                   ]
                   print ""
                  re-update-token
                  initialize-change-vector
                  clear-links
                  clear-drawing
                  set listFrom[]
                  set listTo[]
                  (foreach ?1 ?2 [
                    ask (turtle ((read-from-string ?1) - 1))[
                      give-out-token ((read-from-string ?1) - 1) (turtle ((read-from-string ?2) - 1))
                    ]
                  ])
                  update-token
                  tick
                  file-close
                  file-open file2
                  file-print (word (item (totalGameCount - 1) GameName)", "trialCount", " roundCount ", " averageRank " , "(( CorrectCount / length seqNoList) * 100 ) ", " fairnessWeight ", "reciprocityWeight ", " groupWeight ", "selfInterestWeight", "vicinityWeight", "randomWeight)
                  file-close
                  
                  set roundCount roundCount + 1
                ])
                set trialCount trialCount + 1
              ])
            ]
          ]
        ]
      ]
    ]
  ]
  file-close
end

to compute-rank [ E-matrix ]
 ;print(word "E-matrix    " E-matrix)
  Let rankMatrixList matrix:to-column-list E-matrix
  ; print(word "rankMatrixList    " rankMatrixList)
  let countPos 0
  foreach rankMatrixList [
    let counter1 1
    repeat length ?[
      let maximum max ?
      let counter2 0
      repeat length ?[
        if (maximum = (item counter2 ?) and maximum > -1)[
          ask turtle countPos[ set rankList replace-item counter2 rankList counter1]
          set ? replace-item counter2 ? -1
        ]
        set counter2 counter2 + 1
      ]
      set counter1 counter1 + 1
    ]
   
   set countPos countPos + 1
  ]
end


to-report string:split[n M]
  let counter 0
  let stringToList[]
  let ACC[]
  repeat length n [
    let counter1 0
    repeat length M[
      if(item counter n = item counter1 M)[
        set ACC lput counter1 ACC 
      ]
      set counter1 counter1 + 1
    ]
    set counter counter + 1
  ]
  set ACC sort ACC
  let cnt 0
  if(length ACC > 0)[
    set stringToList lput substring M 0 (item 0 ACC) stringToList
    repeat ((length ACC) - 1)[
      set stringToList lput substring M ((item cnt ACC) + 1) (item (cnt + 1) ACC) stringToList
      set cnt cnt + 1
    ]
    set stringToList lput (word (substring M ((item cnt ACC) + 1) (length M - 1)) (length M - 1)) stringToList
  ]
  report stringToList
end
@#$#@#$#@
GRAPHICS-WINDOW
194
10
1016
563
-1
-1
1.0
1
12
1
1
1
0
0
0
1
0
811
-521
0
1
1
1
ticks
30.0

SWITCH
8
381
180
414
show-token?
show-token?
0
1
-1000

BUTTON
5
427
75
460
setup
clear-all\nfile-read-for-conversion user-file\n;ifelse(number-of-files = 0)[\n;user-message(\"NO CONVERSION MADE\")][\n;user-message ( word \"FILES SUCCESSFULLY CONVERTED!!\\n\"  number-of-files \" OUTPUT FILES OBTAINED\")]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
11
81
183
114
fairness-weight
fairness-weight
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
10
168
182
201
group-weight
group-weight
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
10
211
182
244
self-interest-weight
self-interest-weight
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
11
124
183
157
reciprocity-weight
reciprocity-weight
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
10
253
182
286
vicinity-weight
vicinity-weight
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
10
295
182
328
random-value
random-value
0
100
1
1
1
NIL
HORIZONTAL

BUTTON
83
429
178
462
go
go1
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

SWITCH
10
338
175
371
Random-self-given?
Random-self-given?
0
1
-1000

SWITCH
39
38
156
71
Set-range?
Set-range?
1
1
-1000

TEXTBOX
1079
14
1279
41
Range of weigths
22
0.0
1

TEXTBOX
1023
36
1321
80
Use when the set-range? switch is on
18
0.0
1

TEXTBOX
1019
62
1065
87
Start
20
0.0
1

TEXTBOX
1156
62
1192
87
End
20
0.0
1

SLIDER
995
97
1113
130
S-fairness
S-fairness
0
100
11
1
1
NIL
HORIZONTAL

SLIDER
1118
97
1236
130
E-fairness
E-fairness
S-fairness
100
12
1
1
NIL
HORIZONTAL

SLIDER
996
143
1114
176
S-reciprocity
S-reciprocity
0
100
36
1
1
NIL
HORIZONTAL

SLIDER
1119
143
1235
176
E-reciprocity
E-reciprocity
S-reciprocity
100
50
1
1
NIL
HORIZONTAL

SLIDER
995
190
1115
223
S-group
S-group
0
100
15
1
1
NIL
HORIZONTAL

SLIDER
1120
192
1237
225
E-group
E-group
S-group
100
20
1
1
NIL
HORIZONTAL

SLIDER
995
238
1116
271
S-self-interest
S-self-interest
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
1121
240
1240
273
E-self-interest
E-self-interest
S-self-interest
100
3
1
1
NIL
HORIZONTAL

SLIDER
996
287
1116
320
S-vicinity
S-vicinity
0
100
10
1
1
NIL
HORIZONTAL

SLIDER
1121
287
1239
320
E-vicinity
E-vicinity
S-vicinity
100
10
1
1
NIL
HORIZONTAL

SLIDER
996
335
1119
368
S-random
S-random
0
100
17
1
1
NIL
HORIZONTAL

SLIDER
1121
334
1240
367
E-random
E-random
S-random
100
22
1
1
NIL
HORIZONTAL

SLIDER
1240
98
1338
131
I-fairness
I-fairness
0
100
1
1
1
NIL
HORIZONTAL

TEXTBOX
1237
61
1332
86
Increment
20
0.0
1

SLIDER
1241
143
1342
176
I-reciprocity
I-reciprocity
0
100
10
1
1
NIL
HORIZONTAL

SLIDER
1242
190
1341
223
I-group
I-group
0
100
2
1
1
NIL
HORIZONTAL

SLIDER
1243
239
1344
272
I-self-interest
I-self-interest
0
100
5
1
1
NIL
HORIZONTAL

SLIDER
1241
287
1346
320
I-vicinity
I-vicinity
0
100
2
1
1
NIL
HORIZONTAL

SLIDER
1242
335
1347
368
I-random
I-random
0
100
3
1
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="true">
    <setup>setup1</setup>
    <go>go</go>
    <timeLimit steps="5"/>
    <metric>tokenFromInGroup</metric>
    <metric>tokenFromOutGroup</metric>
    <metric>tokenToInGroup</metric>
    <metric>tokenToOutGroup</metric>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 -270 585
Line -7500403 true 150 150 570 570

@#$#@#$#@
0
@#$#@#$#@
